package com.hh.arome.common.scan.idcard.manymodels;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Base64;
import android.util.Log;

import com.donsee.bean.IDCardInfo;
import com.donsee.bean.SciCardInfo;
import com.donsee.devices.CardReader;
import com.eidlink.serial_idocr.bean.IdInfoBean;
import com.hh.arome.MyApplication;
import com.hh.arome.basecard.BaseSocialSecurityCardBean;
import com.hh.arome.basecard.SocialSecurityCardConfig;
import com.hh.arome.common.scan.idcard.abstractidcard.IIDCardScan;
import com.hh.arome.common.scan.idcard.bean.IDCardInfoBean;
import com.hh.arome.common.scan.idcard.callback.IDCardInitResultCallback;
import com.hh.arome.common.scan.idcard.callback.IDCardScanResultCallback;
import com.hh.arome.lib_nfc.DX_NFCCardReader;
import com.hh.arome.lib_sscard.DX_SSCardReader;
import com.hh.arome.utlis.log.AppLogUtils;
import com.hh.lib_dx_base_multi.DxCardReader;
import com.hh.lib_dx_base_multi.DxInitListener;

import java.io.ByteArrayOutputStream;

/**
 * Description: $东信多合一
 *
 * @author xwh
 * time 2024/9/11
 **/
public class DxMultiIDCardScan implements IIDCardScan {
    private static volatile DxMultiIDCardScan instance = null;


    private IDCardScanResultCallback idCardScanResultCallback;
    long beginTime=0;





    //提供一个公共的访问方法，用于获取该类的唯一实例
    public static DxMultiIDCardScan getInstance(Context context) {
        //如果instance为空，就进行实例化
        if (instance == null) {
            //保证多线程下只有一个线程进行实例化
            synchronized (DxMultiIDCardScan.class) {
                //第二次判断，避免多线程下创建多个实例
                if (instance == null) {
                    instance = new DxMultiIDCardScan();
                }
            }
        }
        return instance;
    }

    @Override
    public void initIDCard(IDCardInitResultCallback idCardInitResultCallback) {
        DxCardReader.getInstance().init(MyApplication.getContext(), new DxInitListener() {
            @Override
            public void onLogInfo(String msg) {

            }

            @Override
            public void initSuccess() {
                if(idCardInitResultCallback!=null){
                    idCardInitResultCallback.onIDCardInitSuccess();
                }

            }

            @Override
            public void initFailed(String code, String msg) {
                if(idCardInitResultCallback!=null){
                    idCardInitResultCallback.onIDCardInitError(code,"身份证初始化失败");
                }
            }
        });
    }

    @Override
    public void startIDCardScan(IDCardScanResultCallback idCardScanResultCallback) {
        this.idCardScanResultCallback=idCardScanResultCallback;
        startRead();
    }



    /**
     *
     * desc:启动轮询社保卡
     * @author xia weihao
     * @date 2024/6/26
     *
     */
    public void startRead() {
        DxCardReader.getInstance().setIdCardListener(new DxCardReader.IDCardListener() {
            @Override
            public void resault(long result, IDCardInfo idCardInfo) {
                if (idCardScanResultCallback != null) {
                    if (result == 0) {
                        handleIDCardData(DxCardReader.getInstance().getCardReader(), idCardInfo);
                    }else {
                        idCardScanResultCallback.onIDCardError(result+"","读身份证失败");
                    }
                }
            }
        });

    }

    /**
     *
     * desc:停止轮询读取社保卡
     * @author xia weihao
     * @date 2024/6/26
     *
     */
    public void stopRead() {
        DxCardReader.getInstance().setIdCardListener(null);
    }



    private void handleIDCardData(  CardReader cardReader,final IDCardInfo idcard ) {

        IDCardInfoBean idCardInfoBean = new IDCardInfoBean();
        idCardInfoBean.setIdcardName(idcard.getName());
        idCardInfoBean.setIdcardNumber(idcard.getIdNO());
        idCardInfoBean.setSex(idcard.getSex());
        idCardInfoBean.setAddress(idcard.getAddress());
        idCardInfoBean.setBirth(idcard.getBirthDate());
        idCardInfoBean.setNation(idcard.getNation());
        idCardInfoBean.setSigningOrganization(idcard.getOrgans());
        idCardInfoBean.setBeginTime(idcard.getIssueDate());
        idCardInfoBean.setEndTime(idcard.getExpireDate());

        String bmpPhotos = "";
        Bitmap image= null;
        try {

             image = cardReader.getBMPByWlt(idcard.getPhoto());
            if (image == null) {
                AppLogUtils.i("东信多合一身份证照片解码null ");
            } else {
                bmpPhotos = bitmapToString(image);
//                AppLogUtils.i("新中新身份证照片解码有值bmpPhotos--" +bmpPhotos);
                AppLogUtils.i("东信多合一身份证照片解码有值bmpPhotos--");
                idCardInfoBean.setPhotoBase64(bmpPhotos);
            }
        } catch (Exception e) {
            e.printStackTrace();
            AppLogUtils.i("东信多合一身份证照片解码异常 " + e);
        } finally {
            if (image != null && !image.isRecycled()) {
                image.recycle();
                image = null;
            }
        }
        if (idCardScanResultCallback != null) {
            idCardScanResultCallback.onIDCardScanSuccess(idCardInfoBean);
        }

    }

    //将Bitmap转换成字符串
    public static String bitmapToString(Bitmap bitmap) {
        try {
            String string = null;
            ByteArrayOutputStream bStream = new ByteArrayOutputStream();
            bitmap.compress(Bitmap.CompressFormat.PNG, 100, bStream);
            byte[] bytes = bStream.toByteArray();
            string = Base64.encodeToString(bytes, Base64.DEFAULT);
            return string;
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }
    @Override
    public void resumeIDCardScan() {

    }

    @Override
    public void pauseIDCardScan() {

    }

    @Override
    public void stopIDCardScan() {

    }

    @Override
    public void destoryIDCardScan() {
        stopRead();
        DxCardReader.getInstance().release(MyApplication.getContext());
    }
}
