package com.hh.arome.storageface;


import static com.hh.arome.common.Constants.Ampe.ACTION_DEVICE_CONFIG;
import static com.hh.arome.common.Constants.Ampe.ACTION_STORAGE_FACE_INFO;
import static com.hh.arome.common.Constants.LogcatValue.LINE;
import static com.hh.arome.common.Constants.LogcatValue.NODE;
import static com.hh.arome.common.Constants.LogcatValue.WARP;
import static com.hh.arome.common.Constants.Value.CAMERA_BRAND;

import android.os.Bundle;
import android.text.TextUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alipay.arome.aromecli.AromeServiceInvoker;
import com.alipay.iot.bpaas.api.BPaaSApi;
import com.alipay.iot.bpaas.api.service.BPaaSCallback;
import com.alipay.iot.bpaas.api.service.BPaaSResponse;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.hh.arome.bean.ampe.AmpeAppletRequestBean;
import com.hh.arome.bean.ampe.AmpeAppletResultBean;
import com.hh.arome.common.ConfigData;
import com.hh.arome.storageface.bean.AmpeResultOpenRocketBean;
import com.hh.arome.storageface.bean.BPaaSOpenRocketBean;
import com.hh.arome.utlis.log.AppLogUtils;
import com.hh.arome.utlis.log.LogType;
import com.hh.arome.utlis.log.SaveLogUtils;

/**
 * Description:支付宝刷脸入库服务
 * @author xwh
 * time 2024/7/18
 **/
public class ZfbBPaaSOpenRocketUtils {
    
    private static final String TAG = ZfbBPaaSOpenRocketUtils.class.getName();

    
    
    
    /**
     * 
     * Description: 启动支付宝 "BPaaSOpenRocket服务
     * @author xwh
     * Time 2024/7/18
     *
    **/
    public static String startZfbBpaasOpenRocketService(BPaaSOpenRocketBean startBPaaSOpenRocketBean, AmpeAppletRequestBean ampeAppletRequestBean,BPaaSOpenRocketListener listener) {
        if (startBPaaSOpenRocketBean == null) {
            AppLogUtils.e("startBPaaSOpenRocketBean:  " + startBPaaSOpenRocketBean);
            return "";
        }

        Bundle bundle = new Bundle();
        // 启动组件的appId

        bundle.putString("schema", "engine://platformapi/startapp?appId=2021003197687001&type=cube");
        JSONObject launchParams = new JSONObject();
        launchParams.put("orgOutId", startBPaaSOpenRocketBean.getOrgOutId());
        launchParams.put("componentOutId", startBPaaSOpenRocketBean.getComponentOutId());
        launchParams.put("isvPid", startBPaaSOpenRocketBean.getIsvPid()); //企业的支付宝的PID
        launchParams.put("appId", startBPaaSOpenRocketBean.getServiceAppId());


        if(!TextUtils.isEmpty(startBPaaSOpenRocketBean.getD2Config())){
            launchParams.put("extInfo", startBPaaSOpenRocketBean.getD2Config());
        }
        String cameraBrand = ConfigData.getDeviceConfigResultBean().getCameraBrand();
        if (CAMERA_BRAND.equals(cameraBrand)) {
            AppLogUtils.e("是马里奥的，不配置zolozConfig");
        } else {
            AppLogUtils.e("非马里奥的，开始配置zolozConfig");
            //各设备 zolozConfig 参数，参考 zolozConfig
            try {
                launchParams.put("zolozConfig",startBPaaSOpenRocketBean.getZolozConfig());
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        if (!TextUtils.isEmpty(startBPaaSOpenRocketBean.getUniqueId().trim())) {
          //  launchParams.put("ext", "xxxx"); // 刷脸授权开通时非必传，刷身份证和手工录入开通时不能传该参数
            launchParams.put("uniqueId", startBPaaSOpenRocketBean.getUniqueId()); // 刷脸授权开通时不能传该参数，刷身份证开通和手工录入开通时必传

            JSONObject identityInfoJson = new JSONObject();
            identityInfoJson.put("name", startBPaaSOpenRocketBean.getName());
            identityInfoJson.put("certNo", startBPaaSOpenRocketBean.getCertNo());
            if (!TextUtils.isEmpty(startBPaaSOpenRocketBean.getPhone())) {
                identityInfoJson.put("phone", startBPaaSOpenRocketBean.getPhone());  //选填
            }
            JSONObject businessParamsJson = new JSONObject();
            businessParamsJson.put("identityInfo", identityInfoJson );
            launchParams.put("businessParams", businessParamsJson );
        }
        bundle.putString("launchParams", launchParams.toJSONString());
        bundle.putString("controlParams", "{\"isShowTitleBar\":false, \"showLoading\": false}");
        //保存日志
        SaveLogUtils.saveLog("BPaaSOpenRocket start","abcpAppId:"+startBPaaSOpenRocketBean.getMachineAppId()+"-----"+bundle.toString());
        AppLogUtils.e("---------------123  BPaaSOpenRocket start"+"abcpAppId:"+startBPaaSOpenRocketBean.getMachineAppId()+"-----"+bundle.toString());
        String sessionId = BPaaSApi.getInstance().startBPaaSService(startBPaaSOpenRocketBean.getMachineAppId(), "BPaaSOpenRocket",
                bundle, new BPaaSCallback() {
                    @Override
                    public void onResponse(BPaaSResponse bPaaSResponse) {
                        AppLogUtils.e("---------------123  startBPaaSService -onResponse "+bPaaSResponse.toString());

                        if (bPaaSResponse != null && bPaaSResponse.getCode() == BPaaSResponse.RESULT_CODE_SUCCESS) {
                            // bpaas服务调用成功，不需要处理
                        } else {
                            // bpaas服务调用失败，可以重试
                        }
                    }

                    @Override
                    public void onEvent(String eventId, String data, Bundle bundle) {
                        // 接收到组件消息，处理结果
                        AppLogUtils.d(TAG+ "---------------123  BPaaSOpenRocket onEvent: eventId=" + eventId + ",data=" + data + ",extra=" + bundle);
                        try {
                             boolean isSuccess=false;
                            JSONObject dataJson = JSON.parseObject(data);
                            if (eventId.equals("BPaaSOpenRocket") && dataJson.getString("eventType").equals("rocketResult")) {
                                JSONObject eventData = dataJson.getJSONObject("eventData");
                                String subCode = eventData.getString("subCode");
                                // 开通成功
                                if (subCode.equals("SUCCESS")) {
                                    isSuccess=true;
                                }
                                // 刷脸授权开通时若授权失败，则在接收组件消息中获取uniqueId，后续传入相应组件编码和uniqueId切换至刷身份证或者手工录入开通
                                if (subCode.equals("90001")) {
                                    AppLogUtils.d(TAG, eventData.getString("uniqueId"));
                                    isSuccess=false;
                                }
                                // 其他都是开通失败

                            }

                            AmpeResultOpenRocketBean ssConfigBean = new AmpeResultOpenRocketBean();
                            ssConfigBean.setAction(ACTION_STORAGE_FACE_INFO);
                            ssConfigBean.setEvent(ampeAppletRequestBean.getEvent());
                            ssConfigBean.setTaskId(ampeAppletRequestBean.getTaskId());
                            ssConfigBean.setSuccess(isSuccess);
                            ssConfigBean.setMessage(isSuccess?"成功":"失败");
                            ssConfigBean.setData(dataJson.toJSONString());
                            JsonObject jsonObject = new Gson().toJsonTree(ssConfigBean).getAsJsonObject();
                            if (listener != null) {
                                listener.callback(jsonObject);
                            }

                        } catch (JSONException e) {
                            AppLogUtils.e(e.toString());
                        }
                    }
                });
        return sessionId;
    }



    private String getLaunchParams() {

        /**
         * @程鹏 辛苦帮忙签约下机构编码和组件编码
         * 机构编码： HangHuiJingQuTest
         * 组件编码
         * 授权：day_label_sign_online
         * 刷证的：device_common_verify_import
         */

        JSONObject launchParamsJson = new JSONObject();
        //机构编码，示例：abcdefg_123
        launchParamsJson.put("orgOutId", "hanghuijingquTest3"); //"SCENIC_5155110950975552"
        //组件编码，示例：verify_test
        launchParamsJson.put("componentOutId", "day_label_sign_online");//
        //ISV的PID，示例：2088202967380463
        launchParamsJson.put("isvPid", "2088721034139660");//航汇的isvPid是 2088721034139660
        //launchParamsJson.put("appId", ConfigManager.machineAppId);
        launchParamsJson.put("appId", "2021003181682188");//测试支付宝小程序 APPID

        //设备 zolozConfig 参数
  //      launchParamsJson.put("zolozConfig", getHardWareConfig());





/*
        launchParamsJson.put("uniqueId", "123456");

        JSONObject identityInfoJson = new JSONObject();
        identityInfoJson.put("name", "曾英皓");
        identityInfoJson.put("certNo", "330821199603150212");

        JSONObject businessParamsJson = new JSONObject();
        businessParamsJson.put("identityInfo ", identityInfoJson.toJSONString() );

        launchParamsJson.put("businessParams", businessParamsJson.toJSONString());
        */

        AppLogUtils.i(true, "getLaunchParams", "launchParamsJson= " + launchParamsJson.toString());

        return launchParamsJson.toString();

    }







    public static void stopBpaasService(String mSessionId) {
        if ( TextUtils.isEmpty(mSessionId)) {
            return;
        }
        AppLogUtils.e("stopBpaasService--SessionId =" + mSessionId);
        BPaaSApi.getInstance().stopBPaaSService(ConfigData.getDeviceConfigResultBean().getAbcpAppId(), "BPaaSOpenRocket", new Bundle(), mSessionId, new BPaaSCallback() {
            @Override
            public void onResponse(BPaaSResponse bPaaSResponse) {
                AppLogUtils.e("停止回调onResponse");
                if (bPaaSResponse == null || bPaaSResponse.getCode() == BPaaSResponse.RESULT_CODE_FAIL) {
                    AppLogUtils.e("stopBPaaSService--onResponse--失败 stopBPaaSService bPaaSResponse=" + bPaaSResponse);
                    return;
                }
                String subCode = bPaaSResponse.getSubCode();
                int code = bPaaSResponse.getCode();
                AppLogUtils.e("stopBPaaSService--onResponse--成功 stopBPaaSService subCode=" + subCode + "code：" + code);
            }

            @Override
            public void onEvent(String s, String resultStr, Bundle bundle) {
                AppLogUtils.e("stopBPaaSService--onEvent--resultStr:" + resultStr);
            }
        });
    }

    public interface BPaaSOpenRocketListener{
        public void callback(JsonObject jsonObject);
    }

}
