package com.hh.lib_dx_base_multi;

import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import com.common.lib_base.CommonApplication;
import com.donsee.bean.IDCardInfo;
import com.donsee.bean.SciCardInfo;
import com.donsee.devices.CardReader;
import com.donsee.devices.LogUtils;
import com.hh.arome.basecard.BaseSocialSecurityCardBean;
import com.hh.arome.basecard.SocialSecurityCardConfig;
import com.hh.arome.basecard.listener.SocialSecurityCardResultListener;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * Description: $
 *
 * @author xwh
 * time 2024/9/11
 **/
public class DxCardReader {

    private volatile static DxCardReader instance;
    private DxInitListener listener;

    private Handler mHandler;
    private UsbDevice usbDevice;
    private UsbManager usbManager;
    private PendingIntent mPendingIntent;

    //硬件厂商回复：多合一免驱的，都是这个固定的vid pid
    private int pid = 57407;
    private int vid = 1742;
    private final String ACTION_USB_PERMISSION = "com.hh.lib_dx_base_multi.USB_PERMISSION";
    long result = -1;

    private USBReceiver mUsbReceiver;

    private CardReader cardReader;

    private volatile InitStatusEnum statusEnum = InitStatusEnum.DEFAULT;//设备是否初始化

    private static final int MESSAGE_RETRY_READER = 0x99; //重连

    public static DxCardReader getInstance() {
        if (instance == null) {
            synchronized (DxCardReader.class) {
                if (instance == null) {
                    instance = new DxCardReader();

                }
            }
        }
        return instance;
    }

    public synchronized void init(Context context, DxInitListener listener) {
        this.listener = listener;
        if(cardReader==null){
            synchronized (DxCardReader.class){
                if(cardReader==null){
                    cardReader = new CardReader(CommonApplication.getContext());
                    initUSB(CommonApplication.getContext());
                    initCardReader(context,listener);
                }
            }
        }
    }

    private void initCardReader(Context context, DxInitListener listener){
        try {
            result = cardReader.Donsee_Open("USB");
            if (result == 0) {
                if (listener != null && statusEnum != InitStatusEnum.SUCCESS) {
                    statusEnum = InitStatusEnum.SUCCESS;
                    listener.initSuccess();
                    startRead();
                }
            } else {
                if (listener != null && statusEnum != InitStatusEnum.FAIL) {
                    statusEnum = InitStatusEnum.FAIL;
                    listener.initFailed(result + "", "社保卡读卡器初始化失败");
                    stopRead();
                }
                getmHandler().removeMessages(MESSAGE_RETRY_READER);
                getmHandler().sendEmptyMessageDelayed(MESSAGE_RETRY_READER,5000);
            }

            registerReceiver(CommonApplication.getContext());
        }catch (Exception e){
            getmHandler().removeMessages(MESSAGE_RETRY_READER);
            getmHandler().sendEmptyMessageDelayed(MESSAGE_RETRY_READER,5000);
        }

    }

    public CardReader getCardReader() {
        return cardReader;
    }

    /**
     * desc:初始化USB
     *
     * @author xia weihao
     * @date 2024/6/26
     */
    private void initUSB(Context mContext) {
        usbManager = (UsbManager) mContext.getApplicationContext().getSystemService(Context.USB_SERVICE);
        if (Build.VERSION.SDK_INT >= 31) {
            mPendingIntent = PendingIntent.getBroadcast(mContext, 0, new Intent(ACTION_USB_PERMISSION), PendingIntent.FLAG_IMMUTABLE);
        } else {
            mPendingIntent = PendingIntent.getBroadcast(mContext, 0, new Intent(ACTION_USB_PERMISSION), 0);
        }
        IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
        mContext.registerReceiver(mUsbReceiver, filter);
        HashMap<String, UsbDevice> deviceList = usbManager.getDeviceList();
        if (!deviceList.isEmpty()) {
            for (UsbDevice next : deviceList.values()) {
                if (next.getVendorId() == vid && next.getProductId() == pid) {
                    usbDevice = next;
                    if (!usbManager.hasPermission(usbDevice)) {
                        usbManager.requestPermission(usbDevice, mPendingIntent);
                    } else {
                        initCardReader(CommonApplication.getContext(),listener);
                    }
                }
            }
        }
    }

    /**
     *
     * @author xwh
     * Time 2024/9/11
     * Description: 是否初始化成功
     *
    **/
    public boolean isInitSuccessed(){
        return statusEnum==InitStatusEnum.SUCCESS;
    }
    /**
     * desc:注册USB监听广播
     *
     * @author xia weihao
     * @date 2024/6/26
     */
    public void registerReceiver(Context mContext) {
        IntentFilter filter = new IntentFilter();
        filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
        filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        mUsbReceiver = new USBReceiver();
        mContext.registerReceiver(mUsbReceiver, filter);
    }


    /**
     * desc:USB拔插广播
     *
     * @author xia weihao
     * @date 2024/6/26
     */
    private class USBReceiver extends BroadcastReceiver {
        private volatile  int usbResylt=0;
        @Override
        public void onReceive(Context context, Intent intent) {
            // 这里可以拿到插入的USB设备对象
            LogUtils.d("intent.getAction = " + intent.getAction());
            UsbDevice receiveDevice = intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
            if (receiveDevice != null) {
                /*    private int pid = 57407;  private int vid = 1742;*/
                if (receiveDevice.getProductId() != pid || receiveDevice.getVendorId() != vid)
                    return;
            }
            switch (intent.getAction()) {
                case UsbManager.ACTION_USB_DEVICE_ATTACHED: // 插入USB设备
                    if (usbResylt == -1) {
                        init(CommonApplication.getContext(),listener);
                    }
                    usbResylt=0;

                    break;
                case UsbManager.ACTION_USB_DEVICE_DETACHED: // 拔出USB设备
                    usbDevice = null;
                    if (usbResylt == 0) {
                        usbDevice = null;
                        stopRead();
                        if(cardReader!=null) {
                            cardReader.Donsee_Close();
                        }
                        cardReader=null;
                        usbResylt = -1;
                    }
                    break;
                case ACTION_USB_PERMISSION:
                    UsbDevice usbDevice1 = (UsbDevice) intent.getParcelableExtra("device");
                    if (intent.getBooleanExtra("permission", false) && usbDevice1 != null) {
                        initCardReader(CommonApplication.getContext(),listener);
                    }
                    break;
                default:
                    break;
            }
        }
    }


    /**
     * desc:销毁资源
     *
     * @author xia weihao
     * @date 2024/6/26
     */
    public void release(Context context) {
        stopRead();
        if (mUsbReceiver != null) {
            context.unregisterReceiver(mUsbReceiver);
        }
        if (cardReader != null) {
            cardReader.Donsee_Close();
            cardReader.unregisterUSBReceiver();
            cardReader=null;
        }
        instance = null;

    }

    public enum InitStatusEnum {
        DEFAULT, FAIL, SUCCESS
    }


    boolean isSSCard = false;
    boolean isIDCard = true;
    private CReaderThread cReaderDataThread;
    private volatile boolean isRuning = false;
    long beginTime=0;

    private class CReaderThread extends Thread {
        private final Object lock = new Object();
        private volatile boolean pause = false;

        public CReaderThread() {
        }

        void pauseThread() {
            pause = true;
        }

        void resumeThread() {
            pause = false;
            synchronized (lock) {
                lock.notifyAll();
            }
        }

        void onPause() {
            synchronized (lock) {
                try {
                    lock.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        @Override
        public void run() {
            super.run();
            while (isRuning) {
                while (pause) onPause();
                try {
                    readIDCard();
                    Thread.sleep(100);
                    readSSCard();
                    Thread.sleep(100);
                    readScanner();
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                    pause=false;
                    isRuning=false;
                    cReaderDataThread=null;
                    e.printStackTrace();
                }

            }
        }
    }

    /**
     *
     * @author xwh
     * Time 2024/9/30
     * Description: 读取身份证
     *
    **/
    private void readIDCard() throws InterruptedException {
        if(!DxCardReader.getInstance().isInitSuccessed()){
            Log.e("DxCardReader","isInitSuccessed =false");
            return;
        }
        final CardReader cardReader=DxCardReader.getInstance().getCardReader();
        if(cardReader==null){
            Log.e("DxCardReader","CardReader = null");
            return;
        }
        IDCardInfo idCardInfo = new IDCardInfo();
        beginTime = System.currentTimeMillis();
        long result = cardReader.Donsee_ReadIDCard(1, idCardInfo);
        if (result == 0) {
            Thread.sleep(100);
            cardReader.Donsee_Beep(50);
            long time = System.currentTimeMillis() - beginTime;
            String readTime =""+time/1000+"."+time%1000+"s";
            if (idCardInfo.certType.equals("0")) {
                idCardInfo.certType = "居民身份证";
            } else if (idCardInfo.certType.equals("I")) {
                idCardInfo.certType = "外国人永久居留证";
            } else if (idCardInfo.certType.equals("J")) {
                idCardInfo.certType = "港澳台居民居住证";
            }
            if(idCardListener!=null){
                idCardListener.resault(0,idCardInfo);
            }

        } else {
            if(idCardListener!=null){
                idCardListener.resault(result,null);
            }
        }
    }


    private IDCardListener idCardListener;

    public void setIdCardListener(IDCardListener idCardListener) {
        this.idCardListener = idCardListener;
    }

    public  interface  IDCardListener{
        void resault(long result, IDCardInfo idCardInfo );
    }



    /**
     *
     * desc:读取社保卡数据
     * @author xia weihao
     * @date 2024/6/26
     *
     */
    private void readSSCard() {
        if(!DxCardReader.getInstance().isInitSuccessed()){
            Log.e("DxCardReader","isInitSuccessed =false");
            return;
        }
        SciCardInfo sciCardInfo = new SciCardInfo();
        //nType 1，	有SAM卡返回全部信息   2，	无SAM卡返回卡号
        byte[] errInfor = new byte[256];
        SocialSecurityCardConfig config = mSSConfig;
        if(config==null){
            config=new SocialSecurityCardConfig();
        }
        CardReader cardReader=DxCardReader.getInstance().getCardReader();
        if(cardReader==null){
            Log.e("DxCardReader","CardReader = null");
            return;
        }
        long result = cardReader.Donsee_ReadSSCard(config.getSlotPSAM(), config.getnType(), sciCardInfo, errInfor);

        if (listener != null && isRuning) {
            if (result == 0) {
                BaseSocialSecurityCardBean info = new BaseSocialSecurityCardBean();
                info.cardNumber = sciCardInfo.cardNO;
                info.name = sciCardInfo.name;
                info.sex = sciCardInfo.sex;
                info.nation = sciCardInfo.nation;
                info.idNumber = sciCardInfo.idNO;
                info.birthDate = sciCardInfo.birthDate;
                info.issueDate = sciCardInfo.issueDate;
                info.expireDate = sciCardInfo.expireDate;
                info.cityCode = sciCardInfo.cityCode;
                info.fullVersion = sciCardInfo.fullVersion;
                if(socialSecurityCardResultListener!=null){
                    socialSecurityCardResultListener.onSuccess(info);
                }

            } else {
                if(socialSecurityCardResultListener!=null) {
                    socialSecurityCardResultListener.onFail("读社保卡失败：" + CardReader.getErrorMessage(result));
                }
            }
        }

    }

    SocialSecurityCardConfig mSSConfig;

    public void setmSSConfig(SocialSecurityCardConfig mSSConfig) {
        this.mSSConfig = mSSConfig;
    }

    SocialSecurityCardResultListener socialSecurityCardResultListener;

    public void setSocialSecurityCardResultListener(SocialSecurityCardResultListener socialSecurityCardResultListener) {
        this.socialSecurityCardResultListener = socialSecurityCardResultListener;
    }


    /**
     *
     * @author xwh
     * Time 2024/9/30
     * Description: 读取二维码
     *
    **/
    private void readScanner() {
        if(!DxCardReader.getInstance().isInitSuccessed()){
            Log.e("DxCardReader", "isInitSuccessed =false");
            return;
        }
        CardReader cardReader=DxCardReader.getInstance().getCardReader();
        if(cardReader==null){
            Log.e("DxCardReader","CardReader = null");
            return;
        }
        beginTime = System.currentTimeMillis();
        int[] len = new int[1];
        byte[] data = new byte[1024];
        long re;
        // 扫码模块，安卓建议扫码模组直接TTL串口接安卓主板更方便。或者用键盘监听模式。
        // 注意扫码有个超时等待入参，单位ms，超时等待时间内是中段的，读卡器无法进行其他读卡工作；
        // 根据不同码的密度，建议最低500~1000ms，时间越长，扫码成功率越高，但是相对整个轮询时间也会越长；
        re = cardReader.Donsee_ReadScannerInfo(1000,  len, data);
        long time = System.currentTimeMillis() - beginTime;
        String readTime =""+time/1000+"."+time%1000+"s";
        if (re == 0 && len[0]>0) {
            byte[] qrcode  = new byte[len[0]];
            System.arraycopy(data,0,qrcode,0,len[0]);
            try {
                String qrCodeStr = new String(DeleteElement(qrcode), "UTF-8");
                if(qrResultListener!=null){
                    qrResultListener.onScanSuccess(qrCodeStr);
                }
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                if(qrResultListener!=null) {
                    qrResultListener.onScanFail(re + "", "不支持的编码");
                }
            }

        }else{
            if(qrResultListener!=null){
                qrResultListener.onScanFail( re+"",CardReader.getErrorMessage(re));
            }
        }
    }

    private QrResultListener qrResultListener;
    public  interface QrResultListener{
        void onScanSuccess(String qrCode);
        void onScanFail(String code,String errMsg);
    }

    public void setQrResultListener(QrResultListener qrResultListener) {
        this.qrResultListener = qrResultListener;
    }

    /**
     *
     * desc:启动轮询社保卡
     * @author xia weihao
     * @date 2024/6/26
     *
     */
    public synchronized void startRead() {

        if(cReaderDataThread!=null&&cReaderDataThread.isAlive()){
            return;
        }
        if(cReaderDataThread!=null){
            isRuning = false;
            cReaderDataThread.interrupt();
            cReaderDataThread = null;
        }
        if (cReaderDataThread == null) {
            cReaderDataThread = new CReaderThread();
        }
        isRuning = true;
        cReaderDataThread.start();

    }

    /**
     *
     * desc:停止轮询读取社保卡
     * @author xia weihao
     * @date 2024/6/26
     *
     */
    public synchronized void stopRead() {
        if (cReaderDataThread == null) {
//            showString("请先启动读卡");
            return;
        }
        isRuning = false;
        cReaderDataThread.pause = false;
        cReaderDataThread.interrupt();
        cReaderDataThread = null;
    }


    public Handler getmHandler() {
        if (mHandler == null) {
            synchronized (DxCardReader.class) {
                if (mHandler == null) {
                    mHandler = new Handler(Looper.getMainLooper()) {
                        @Override
                        public void handleMessage(Message msg) {
                            switch (msg.what) {
                                case MESSAGE_RETRY_READER:
                                    initCardReader(CommonApplication.getContext(),listener);
                                    break;
                                default:
                                    break;
                            }
                            super.handleMessage(msg);
                        }
                    };
                }
            }
        }
        return mHandler;
    }
    /**
     * 数据处理（剔除\r\n的情况）
     */
    public  byte[] DeleteElement(byte[] arg0){
        List<Byte> list=new ArrayList<>();
        for (int i = 0; i < arg0.length; i++) {
            if (arg0[i]!=13&&arg0[i]!=10){
                list.add(arg0[i]);
            }
        }
        byte[] bytes=new byte[list.size()];
        for (int i = 0; i < list.size(); i++) {
            bytes[i]=list.get(i);
        }
        return bytes;
    }
}
